define(["require", "exports", "jssha", "js-sha512", "js-sha3"], function (require, exports, jsSHA, jsSha512, sha3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.bigNumberToBuffer = exports.blake2b256 = exports.keccak256Checksum = exports.blake2b = exports.blake256Checksum = exports.blake256 = exports.sha512_256 = exports.sha256Checksum = exports.sha256x2 = exports.sha256 = exports.toHex = exports.hexStr2byteArray = exports.byteArray2hexStr = exports.numberToHex = void 0;
    var Blake256 = require('./externals/blake256');
    var Blake2B = require('./externals/blake2b');
    var BigNum = require('browserify-bignum');
    function numberToHex(number, length) {
        var hex = number.toString(16);
        if (hex.length % 2 === 1) {
            hex = '0' + hex;
        }
        return hex.padStart(length, '0');
    }
    exports.numberToHex = numberToHex;
    function isHexChar(c) {
        if ((c >= 'A' && c <= 'F') ||
            (c >= 'a' && c <= 'f') ||
            (c >= '0' && c <= '9')) {
            return 1;
        }
        return 0;
    }
    /* Convert a hex char to value */
    function hexChar2byte(c) {
        var d = 0;
        if (c >= 'A' && c <= 'F') {
            d = c.charCodeAt(0) - 'A'.charCodeAt(0) + 10;
        }
        else if (c >= 'a' && c <= 'f') {
            d = c.charCodeAt(0) - 'a'.charCodeAt(0) + 10;
        }
        else if (c >= '0' && c <= '9') {
            d = c.charCodeAt(0) - '0'.charCodeAt(0);
        }
        return d;
    }
    /* Convert a byte to string */
    function byte2hexStr(byte) {
        var hexByteMap = "0123456789ABCDEF";
        var str = "";
        str += hexByteMap.charAt(byte >> 4);
        str += hexByteMap.charAt(byte & 0x0f);
        return str;
    }
    function byteArray2hexStr(byteArray) {
        var str = "";
        for (var i = 0; i < (byteArray.length - 1); i++) {
            str += byte2hexStr(byteArray[i]);
        }
        str += byte2hexStr(byteArray[i]);
        return str;
    }
    exports.byteArray2hexStr = byteArray2hexStr;
    function hexStr2byteArray(str) {
        var byteArray = Array();
        var d = 0;
        var i = 0;
        var j = 0;
        var k = 0;
        for (i = 0; i < str.length; i++) {
            var c = str.charAt(i);
            if (isHexChar(c)) {
                d <<= 4;
                d += hexChar2byte(c);
                j++;
                if (0 === (j % 2)) {
                    byteArray[k++] = d;
                    d = 0;
                }
            }
        }
        return byteArray;
    }
    exports.hexStr2byteArray = hexStr2byteArray;
    function toHex(arrayOfBytes) {
        var hex = '';
        for (var i = 0; i < arrayOfBytes.length; i++) {
            hex += numberToHex(arrayOfBytes[i]);
        }
        return hex;
    }
    exports.toHex = toHex;
    function sha256(payload, format) {
        if (format === void 0) { format = 'HEX'; }
        var sha = new jsSHA('SHA-256', format);
        sha.update(payload);
        return sha.getHash(format);
    }
    exports.sha256 = sha256;
    function sha256x2(buffer, format) {
        if (format === void 0) { format = 'HEX'; }
        return sha256(sha256(buffer, format), format);
    }
    exports.sha256x2 = sha256x2;
    function sha256Checksum(payload) {
        return sha256(sha256(payload)).substr(0, 8);
    }
    exports.sha256Checksum = sha256Checksum;
    function sha512_256(payload) {
        var hash = jsSha512.sha512_256.create();
        hash.update(Buffer.from(payload, "hex"));
        return hash.hex().toUpperCase();
    }
    exports.sha512_256 = sha512_256;
    function blake256(hexString) {
        return new Blake256().update(hexString, 'hex').digest('hex');
    }
    exports.blake256 = blake256;
    function blake256Checksum(payload) {
        return blake256(blake256(payload)).substr(0, 8);
    }
    exports.blake256Checksum = blake256Checksum;
    function blake2b(hexString, outlen) {
        return new Blake2B(outlen).update(Buffer.from(hexString, 'hex')).digest('hex');
    }
    exports.blake2b = blake2b;
    function keccak256Checksum(payload) {
        return sha3.keccak256(payload).toString().substr(0, 8);
    }
    exports.keccak256Checksum = keccak256Checksum;
    function blake2b256(hexString) {
        return new Blake2B(32).update(Buffer.from(hexString, 'hex'), 32).digest('hex');
    }
    exports.blake2b256 = blake2b256;
    function bigNumberToBuffer(bignumber, size) {
        return new BigNum(bignumber).toBuffer({ size: size, endian: 'big' });
    }
    exports.bigNumberToBuffer = bigNumberToBuffer;
});
//# sourceMappingURL=utils.js.map