define(["require", "exports", "./bitcoin_validator"], function (require, exports, BTCValidator) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.isValidAddress = void 0;
    var base32 = require('../crypto/externals/base32');
    var bech32 = require('../crypto/externals/bech32');
    function validateAddress(address, currency, opts) {
        var networkType = opts ? opts.networkType : '';
        var regexp = currency.regexp;
        var prefix = 'bitcoincash';
        var raw_address;
        var res = address.split(':');
        if (res.length === 1) {
            raw_address = address;
        }
        else {
            if (res[0] !== 'bitcoincash') {
                return false;
            }
            raw_address = res[1];
        }
        if (!regexp.test(raw_address)) {
            return false;
        }
        if (raw_address.toLowerCase() != raw_address && raw_address.toUpperCase() != raw_address) {
            return false;
        }
        var decoded = base32.b32decode(raw_address);
        if (networkType === 'testnet') {
            prefix = 'bchtest';
        }
        try {
            if (bech32.verifyChecksum(prefix, decoded, bech32.encodings.BECH32)) {
                return false;
            }
        }
        catch (e) {
            return false;
        }
        return true;
    }
    function isValidAddress(address, currency, opts) {
        return validateAddress(address, currency, opts) || BTCValidator.isValidAddress(address, currency, opts);
    }
    exports.isValidAddress = isValidAddress;
});
//# sourceMappingURL=bch_validator.js.map