define(["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.isValidAddress = void 0;
    var bech32 = require('../crypto/externals/bech32');
    var DEFAULT_NETWORK_TYPE = 'prod';
    // bip 173 bech 32 addresses (https://github.com/bitcoin/bips/blob/master/bip-0173.mediawiki)
    function isValidAddress(address, currency, opts) {
        var networkType = opts ? opts.networkType : DEFAULT_NETWORK_TYPE;
        var decoded = bech32.decode(address, bech32.encodings.BECH32);
        if (!decoded) {
            return false;
        }
        var bech32Hrp = decoded.hrp;
        var correctBech32Hrps;
        if (networkType === 'prod' || networkType === 'testnet') {
            correctBech32Hrps = currency.bech32Hrp[networkType];
        }
        else if (currency.bech32Hrp) {
            correctBech32Hrps = currency.bech32Hrp.prod.concat(currency.bech32Hrp.testnet);
        }
        else {
            return false;
        }
        if (correctBech32Hrps.indexOf(bech32Hrp) === -1) {
            return false;
        }
        return true;
    }
    exports.isValidAddress = isValidAddress;
});
//# sourceMappingURL=bip173_validator.js.map