define(["require", "exports", "../crypto/utils"], function (require, exports, utils_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.isValidAddress = void 0;
    var cnBase58 = require('./crypto/externals/cnBase58');
    var DEFAULT_NETWORK_TYPE = 'prod';
    var addressRegTest = new RegExp('^[123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz]{95}$');
    var integratedAddressRegTest = new RegExp('^[123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz]{106}$');
    function validateNetwork(decoded, currency, networkType, addressType) {
        var network = currency.addressTypes;
        if (addressType === 'integrated') {
            network = currency.iAddressTypes;
        }
        var at = parseInt(decoded.substr(0, 2), 16).toString();
        switch (networkType) {
            case 'prod':
                return network.prod.indexOf(at) >= 0;
            case 'testnet':
                return network.testnet.indexOf(at) >= 0;
            case 'stagenet':
                return network.stagenet.indexOf(at) >= 0;
            case 'both':
                return network.prod.indexOf(at) >= 0 || network.testnet.indexOf(at) >= 0 || network.stagenet.indexOf(at) >= 0;
            default:
                return false;
        }
    }
    function hextobin(hex) {
        if (hex.length % 2 !== 0)
            return null;
        var res = new Uint8Array(hex.length / 2);
        for (var i = 0; i < hex.length / 2; ++i) {
            res[i] = parseInt(hex.slice(i * 2, i * 2 + 2), 16);
        }
        return res;
    }
    function isValidAddress(address, currency, opts) {
        if (opts === void 0) { opts = {}; }
        var _a = opts.networkType, networkType = _a === void 0 ? DEFAULT_NETWORK_TYPE : _a;
        var addressType = 'standard';
        if (!addressRegTest.test(address)) {
            if (integratedAddressRegTest.test(address)) {
                addressType = 'integrated';
            }
            else {
                return false;
            }
        }
        var decodedAddrStr = cnBase58.decode(address);
        if (!decodedAddrStr)
            return false;
        if (!validateNetwork(decodedAddrStr, currency, networkType, addressType))
            return false;
        var addrChecksum = decodedAddrStr.slice(-8);
        var hashChecksum = (0, utils_1.keccak256Checksum)(hextobin(decodedAddrStr.slice(0, -8)));
        return addrChecksum === hashChecksum;
    }
    exports.isValidAddress = isValidAddress;
});
//# sourceMappingURL=monero_validator.js.map