define(["require", "exports", "../crypto/utils", "base-x"], function (require, exports, utils_1, baseX) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.verifyChecksum = exports.isValidAddress = void 0;
    var ALLOWED_CHARS = '13456789abcdefghijkmnopqrstuwxyz';
    var codec = baseX(ALLOWED_CHARS);
    // https://github.com/nanocurrency/raiblocks/wiki/Accounts,-Keys,-Seeds,-and-Wallet-Identifiers
    var regexp = new RegExp('^(xrb|nano)_([' + ALLOWED_CHARS + ']{60})$');
    function isValidAddress(address) {
        if (regexp.test(address)) {
            return verifyChecksum(address);
        }
        return false;
    }
    exports.isValidAddress = isValidAddress;
    function verifyChecksum(address) {
        var regexResult = regexp.exec(address);
        if (regexResult === null) {
            return false;
        }
        else {
            var bytes = codec.decode(regexResult[2]).slice(-37);
            // https://github.com/nanocurrency/raiblocks/blob/master/rai/lib/numbers.cpp#L73
            var computedChecksum = (0, utils_1.blake2b)((0, utils_1.toHex)(bytes.slice(0, -5)), 5);
            var checksum = (0, utils_1.toHex)(bytes.slice(-5).reverse());
            return computedChecksum === checksum;
        }
    }
    exports.verifyChecksum = verifyChecksum;
});
//# sourceMappingURL=nano_validator.js.map