define(["require", "exports", "../crypto/utils"], function (require, exports, utils_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.verifyChecksum = exports.isValidAddress = void 0;
    var isEqual = require('lodash.isequal');
    function hexToBytes(hex) {
        var bytes = [];
        for (var c = 0; c < hex.length; c += 2) {
            bytes.push(parseInt(hex.substr(c, 2), 16));
        }
        return bytes;
    }
    function isValidAddress(address) {
        if (address.length !== 76) {
            // Check if it has the basic requirements of an address
            return false;
        }
        // Otherwise check each case
        return verifyChecksum(address);
    }
    exports.isValidAddress = isValidAddress;
    function verifyChecksum(address) {
        var checksumBytes = address.slice(0, 32 * 2);
        var check = address.slice(32 * 2, 38 * 2);
        var blakeHash = (0, utils_1.blake2b)(checksumBytes, 32).slice(0, 6 * 2);
        return !!isEqual(blakeHash, check);
    }
    exports.verifyChecksum = verifyChecksum;
});
//# sourceMappingURL=siacoin_validator.js.map