define(["require", "exports", "base-x", "crc", "../crypto/utils"], function (require, exports, baseX, CRC, utils_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.isValidAddress = void 0;
    var ALPHABET = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ234567';
    var base32 = baseX(ALPHABET);
    var regexp = new RegExp('^[' + ALPHABET + ']{56}$');
    var ed25519PublicKeyVersionByte = (6 << 3);
    function isValidAddress(address) {
        if (regexp.test(address)) {
            return verifyChecksum(address);
        }
        return false;
    }
    exports.isValidAddress = isValidAddress;
    function swap16(n) {
        var lower = n & 0xFF;
        var upper = (n >> 8) & 0xFF;
        return (lower << 8) | upper;
    }
    function verifyChecksum(address) {
        // based on https://github.com/stellar/js-stellar-base/blob/master/src/strkey.js#L126
        var bytes = base32.decode(address);
        if (bytes[0] !== ed25519PublicKeyVersionByte) {
            return false;
        }
        var computedChecksum = (0, utils_1.numberToHex)(swap16(CRC.crc16xmodem(bytes.slice(0, -2))), 4);
        var checksum = (0, utils_1.toHex)(bytes.slice(-2));
        return computedChecksum === checksum;
    }
});
//# sourceMappingURL=stellar_validator.js.map